/* Copyright (c) 2000-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

/* -*- Mode: JavaScript; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * Komodo's Sleuth results tab control (a tab in the output pane)
 *
 *
 * Dev notes: do things naively initially, then optimize:
 * -- initialize first time on demand
 */

if (typeof(PREFERRED_EOL_MODE) == "undefined") {
    const PREFERRED_EOL_MODE = Components.interfaces.ISciMoz.SC_EOL_LF;
}

if (typeof(ko)=='undefined') {
    var ko = {};
}
if (!('sleuth' in ko)) {
    ko.sleuth = {}; // A namespace
}
(function() {
var sleuthResultsLog = ko.logging.getLogger('sleuthResultsTab');
// sleuthResultsLog.setLevel(ko.logging.LOG_DEBUG);

var _gSleuthSvc = Components.classes["@activestate.com/koSleuthService;1"].
                  getService(Components.interfaces.koISleuthService);

var tree_id = "projectUnitTestsResults";

this.SleuthTabManager_Initialize = function SleuthTabManager_Initialize() {
    // If have not yet initialized the tab, then do so.
    try {
        ko.sleuth.tabManager = new SleuthResultsTabManager();
        ko.sleuth.tabManager.initialize();
        ko.sleuth.tabManager.updateView();
        
    } catch(ex) {
        sleuthResultsLog.exception(ex);
    }
};

var SleuthResultsTabManager = function SleuthResultsTabManager() {
    this._runInProgress = false;
    this.view = null;
    // State of the tab-manager

    // The tab maintains tests for the current project, and
    // the any global tests.

    // These members track the project and document we're working with
    this._project = null;
    this._document = null;

    // This member tracks which project the test-plan is associated
    // with.  It's one of this._project, this._document, or null
    this._currTestPlanContainer = null;
    this._currTestPlanContainerType = 'global';

    // The current test plan name
    this._currTestPlanName = null;
    
    this.detailsView = document.getElementById("testplan-details-view");
    this.detailsView.initWithBuffer("", "Text")
    var scimoz = this.detailsView.scimoz;
    scimoz.viewWS = 1;
    scimoz.visible = 1;
    scimoz.caretLineVisible = 0; // gets in the way of highlighted results
    scimoz.eOLMode = PREFERRED_EOL_MODE;
    scimoz.readOnly = true;
    this.scimozEventistInstance = new this.SciMozEventist(this);
    scimoz.hookEvents(this.scimozEventistInstance);
    scimoz.lexer = scimoz.SCLEX_CONTAINER;
    scimoz.setCommandUpdateTarget(window);
    // Use the first indicator to mark replacement syntax errors.
    scimoz.indicSetStyle(0, scimoz.INDIC_SQUIGGLE);
    scimoz.indicSetFore(0, 0x0000ff);
};

SleuthResultsTabManager.prototype._attributeFromContainer = function(attr, container, containerType) {
    if (typeof(containerType) == "undefined") {
        containerType = this._currTestPlanContainerType;
    }
    if (typeof(container) == "undefined") {
        container = this._currTestPlanContainer;
    }
    return this._attrByContainerType(containerType, attr)(container);
};

SleuthResultsTabManager.prototype._attrByContainerType = function(containerType, attr) {
     switch(containerType) {
     case 'project':
       switch(attr) {
         case 'prefset': return function(container) {
               if (!container) throw new Error("container is empty");
               return container.prefset };
         case 'uri': return function(container) {
               if (!container) throw new Error("container is empty");
               return container.url };
         case 'file': return function(container) {
               if (!container) throw new Error("container is empty");
               return container.getFile() };
       }
       break;
     case 'document':
       switch(attr) {
         case 'prefset': return function(container) { return container.prefs };
         case 'uri': return function(container) {
               if (!container.file) {
                   throw new Error("container.file is empty");
               }
               return container.file.URI
                   };
         case 'file': return function(container) {
               if (!container.file) {
                   throw new Error("container.file is empty");
               }
               return container.file
                   };
       }
       break;
     case 'global':
       switch(attr) {
         case 'prefset': return function() {
               return Components.classes["@activestate.com/koPrefService;1"].
                       getService(Components.interfaces.koIPrefService).prefs;
           }
         case 'uri': return function() { return '' };
       }
       break;
     }
     return function(container) {
         dump("Unexpected args to _attrByContainerType: ("
              + containerType + ", " + attr + ")\n");
         throw new Error("global things aren't test-plan containers");
     }
}

SleuthResultsTabManager.prototype.SciMozEventist = function(tabManager) {
    this.tabManager = tabManager;
};
SleuthResultsTabManager.prototype.SciMozEventist.prototype = {
    onStyleNeeded: function() { return false; },
    onCharAdded: function() { return false; },
    onSavePointReached: function() { return false; },
    onSavePointLeft: function() { return false; },
    onModifyAttemptRO: function() { return false; },
    onKey: function() { return false; },
    onDoubleClick: function() {
        var scimoz = this.tabManager.detailsView.scimoz;
        var currentPos = scimoz.currentPos;
        var line = scimoz.lineFromPosition(currentPos);
        var holder = {};
        scimoz.getLine(line, holder);
        this.tabManager._TreeDetail_processLine(holder.value);
        return false;
    },
    onUpdateUI: function() { return false; },
    onModified: function() {},
    onMacroRecord: function() {  return false; },
    onMarginClick: function() { return false; },
    onNeedShown: function() { return false; },
    // nsISupports interface
    onDwellStart: function() {},
    onDwellEnd: function() {},
    QueryInterface: function(iid) {
        if (iid.equals(Components.interfaces.nsIObserver) ||
            iid.equals(Components.interfaces.nsISupports) ||
            iid.equals(Components.interfaces.ISciMozEvents) ||
            iid.equals(Components.interfaces.koISleuthResultsTabManager)) {
            return this;
        }
        throw Components.results.NS_ERROR_NO_INTERFACE;
    }
};


// There's no inheritance going on -- is this cruft?
SleuthResultsTabManager.prototype.constructor = SleuthResultsTabManager;

SleuthResultsTabManager.prototype.initialize = function() {
    // Should be called in the onload handler for the window containing the
    // find results tab XUL.
    this.view = Components.classes['@activestate.com/koSleuthResultsView;1']
        .createInstance(Components.interfaces.koISleuthResultsView);
    if (!this.view) {
        throw("couldn't create a koSleuthResultsView");
    }

    // Ensure the nsITreeView instance is bound to the <tree>.
    var treeWidget = document.getElementById(tree_id);
    var boxObject = treeWidget.treeBoxObject.
                    QueryInterface(Components.interfaces.nsITreeBoxObject);
    if (boxObject.view == null) {
        // We are in a collapsed state -- we need to force the tree to be
        // visible before we can assign the view to it.
        sleuthResultsLog.info("sleuth manager initialize(): forcing it to show");
        this.show();
    }
    boxObject.view = this.view;

    //XXX: Remove setting up a project here -- use the observer instead.
    // var project = null;
    var globalPrefs = (Components.classes["@activestate.com/koPrefService;1"].
                       getService(Components.interfaces.koIPrefService).prefs);
    
    // Prefset on global prefs: a prefset that maps
    // projectURL to [timeStamp + ":" + testPlanName]
    // Purpose of timeStamp: on initialize, remove any
    // entries that are at least 7 days old.
    // It might make more sense to put this pref in the project prefs,
    // but then the user would have to save the project because
    // we changed prefs -- an invisible change, and it's annoying
    // having an item become dirty due to an indirect action.
    
    if (!globalPrefs.hasPref(_gSleuthSvc.LAST_TESTPLAN_PREFNAME)) {
        globalPrefs.setPref(_gSleuthSvc.LAST_TESTPLAN_PREFNAME,
                            Components.classes["@activestate.com/koPreferenceSet;1"].createInstance());
    }
    /*
    if (globalPrefs.hasStringPref(_gSleuthSvc.LAST_TESTPLAN_PROJECT_URL)) {
        var projectURL = globalPrefs.getStringPref(_gSleuthSvc.LAST_TESTPLAN_PROJECT_URL);
        if (projectURL) {
            project = ko.projects.manager.getProjectByURL(projectURL);
        }
    }
    */
    // Sweep out dated _gSleuthSvc.LAST_TESTPLAN_PREFNAME prefs
    try {
        var recentTestplanPrefs = globalPrefs.getPref(_gSleuthSvc.LAST_TESTPLAN_PREFNAME);
        var now = (new Date()).valueOf();
        var dated = 1000 * 3600 * 24 * 30; // 1 month of milliseconds
        var ids = {};
        recentTestplanPrefs.getPrefIds(ids, {});
        ids = ids.value;
        for (var i = 0; i < ids.length; i++) {
            var timestamp = parseInt(recentTestplanPrefs.getStringPref(ids[i]).split(":", 1));

            if (now - timestamp > dated) {
                recentTestplanPrefs.deletePref(ids[i]);
            }
        }
    } catch(ex) {
        // dump("Problem sweeping old testplan prefs:\n");
        log.exception(ex);
    }
    /*
    this._finishSettingUpProject(project, globalPrefs);
    */
    globalPrefs.prefObserverService.addObserver(this, 'select_document_testplan', false);
    globalPrefs.prefObserverService.addObserver(this, 'select_project_testplan', false);
    globalPrefs.prefObserverService.addObserver(this, 'select_global_testplan', false);
    var obSvc = Components.classes["@mozilla.org/observer-service;1"].
        getService(Components.interfaces.nsIObserverService);
    obSvc.addObserver(this, 'current_project_changed', false);
    obSvc.addObserver(this, 'current_testplan_changed', false);
    document.getElementById("projectUnitTestsResults_tabpanel-run-button").
        setAttribute("disabled", "true");
    var self = this;
    this.handle_current_view_changed = function(event) {
        self._useCurrentView();
    };
    window.addEventListener("current_view_changed",
                            this.handle_current_view_changed, false);
    treeWidget.addEventListener("keypress",
                                function(event) {
                                    return self._onKeyPress(event);
                                }, false);
    
    this.starts_with_bracket_re = /^\s*\[(.*)/;
    this._useCurrentView();
    this._harness_indices = this._harness_info_from_language = null;

    var me = this;
    // removeListener also acts as a flag to catch notifications
    // after the containing window has been closed.
    this.removeListener = function() { me.finalize(); }
    ko.main.addWillCloseHandler(this.finalize, this);
};

SleuthResultsTabManager.prototype.finalize = function() {
    if (!this.removeListener) return;
    this.removeListener = null;
    window.removeEventListener("current_view_changed",
                               this.handle_current_view_changed, false);

    var globalPrefs = (Components.classes["@activestate.com/koPrefService;1"].
                       getService(Components.interfaces.koIPrefService).prefs);
    globalPrefs.prefObserverService.removeObserver(this, 'select_document_testplan');
    globalPrefs.prefObserverService.removeObserver(this, 'select_project_testplan');
    globalPrefs.prefObserverService.removeObserver(this, 'select_global_testplan');
    var obSvc = Components.classes["@mozilla.org/observer-service;1"].
        getService(Components.interfaces.nsIObserverService);
    obSvc.removeObserver(this, 'current_project_changed');
    obSvc.removeObserver(this, 'current_testplan_changed');
    // Cleanup the scintilla details view.
    var scimoz = this.detailsView.scimoz;
    scimoz.unhookEvents(this.scimozEventistInstance);
    this.scimozEventistInstance = null;
    scimoz.setCommandUpdateTarget(null);
}

SleuthResultsTabManager.prototype._getCurrentDocument = function() {
    try {
        return ko.views.manager.currentView.koDoc;
    } catch(ex) {
        return null;
    }
};

SleuthResultsTabManager.prototype._useCurrentView = function() {
    // Do this in a setTimeout because otherwise the currentView
    // is still pointing at the view we're moving away from.
    setTimeout(function(self) {
            if (!this.removeListener) {
                // the window was closed between receiving the notification
                // and the timer firing. 
                // Are we getting notifications from other windows?
                return;
            }
            var document = self._getCurrentDocument();
            if (document == self._document) {
                // dump("doc didn't change\n");
                return;
            }
            self._switchToDocument(document);
            self.updateTestPlanMenu();
        }, 0, this);
};

SleuthResultsTabManager.prototype.observe = function observe(subject, topic, data) {
    if (topic == "current_testplan_changed") {
        this.updateTestPlanMenu();
    } else if (topic == 'current_project_changed') {
        var project = ko.projects.manager.getCurrentProject();
        if (project == this._project) {
            return;
        }
        this._switchToProject(project);
        this.updateTestPlanMenu();
    } else if (topic == 'select_project_testplan') {
        // The user created a new test plan on a project
        this.setProject(ko.projects.manager.getCurrentProject());
    } else if (topic == 'select_document_testplan') {
        // The user created a new test plan on a document
        this.setDocument(this._getCurrentDocument());
    } else if (topic == 'select_global_testplan') {
        this.setProject(null);
    }
};

SleuthResultsTabManager.prototype.show = function()
{
    ko.uilayout.ensureOutputPaneShown(window);

    // switch to proper tab
    var tabsWidget = document.getElementById("output_tabs");
    var tabWidget = document.getElementById("projectUnitTestsResults_tab");
    tabsWidget.selectedItem = tabWidget;
}

SleuthResultsTabManager.prototype.selectTest = function(name, cwd, language, command_line) {
    this._currTestPlanName = name;
    this._cwd = cwd;
    this._language = language;
    this._command_line = command_line;
    document.getElementById("sleuthResults-menulist-testPlan").
            setAttribute('label', name);
};

SleuthResultsTabManager.prototype.runTests = function(name, cwd, language, command_line) {
    this.selectTest(name, cwd, language, command_line);
    this._doRunTests();
};


SleuthResultsTabManager.prototype.rerunTests = function() {
    this._doRunTests();
};

SleuthResultsTabManager.prototype._doRunTests = function() {
    this.show();
    var globalPrefs = (Components.classes["@activestate.com/koPrefService;1"].
                       getService(Components.interfaces.koIPrefService).prefs);
    var containerID = (this._currTestPlanContainerType != "global"
                       ? this._attributeFromContainer('uri')
                       : "");
    var containerPrefs =
            globalPrefs.getPref(_gSleuthSvc.LAST_TESTPLAN_PREFNAME);
    containerPrefs.setStringPref(containerID,
                                 ((new Date()).valueOf()
                                  + ":"
                                  + this._currTestPlanName));
    this.clearResults();
    _gSleuthSvc.run_sleuth_tests(this._cwd, this._language, this._command_line,
                                 this, this.view);
};

SleuthResultsTabManager.prototype.editCurrentTestPlan = function()
{
    if (!this._currTestPlanName) {
        // dump("hit SleuthResultsTabManager.editCurrentTestPlan, should be disabled");
        document.getElementById("projectUnitTestsResults_tabpanel-editTestPlan-button").setAttribute("disabled", "true");
        return;
    }
    // Have the dialog work with the testplan set
    var sleuthPrefs = this._getSleuthPrefs();
    try {
        var testPlan = sleuthPrefs.getPref(this._currTestPlanName);
    } catch(ex) {
        dump("this._currTestPlanContainerType = "
             + this._currTestPlanContainerType
             + "\n");
        dump("this._currTestPlanContainer name = "
             + (this._currTestPlanContainer.name
                || this._currTestPlanContainer.displayPath
                || "??")
             + "\n");
        log.exception("editCurrentTestPlan failed: "
                      + ex);
        return;
    }
        
    var obj = {value:false}
    window.openDialog("chrome://komodo/content/sleuth/editTestPlan.xul",
                      "_blank",
                      "chrome,modal,titlebar,resizable",
                      ko, this._currTestPlanName, testPlan, obj);
    if (obj.value) {
        // Update the prefs and the current values
        this._setupTestPlanArgumentsFromPrefs(testPlan);
        ko.statusBar.AddMessage("Updated test plan '" + this._currTestPlanName + "'.",
                                'hotspot', 2000, true);
    }
};

SleuthResultsTabManager.prototype.clearResults = function()
{
    // Clear the list of results.

    // Clear the UI.
    this.view.Clear(0);
    if (this.detailTreeView) {
        this.detailTreeView.Clear();
    }
    this._setDetailsText("");
    this.updateResults(0, 0, 0);
}

SleuthResultsTabManager.prototype._setDetailsText = function(text) {
    var scimoz = this.detailsView.scimoz;
    scimoz.readOnly = false;
    scimoz.text = text;
    scimoz.readOnly = true;
}

SleuthResultsTabManager.prototype.stopUnitTests = function()
{
    _gSleuthSvc.stop_sleuth_tests();
    this.runFinished(false);
}

SleuthResultsTabManager.prototype._checkRerunSettings = function() {
    if (!("_cwd" in this)) return false;
    if (!("_language" in this)) return false;
    return this._cwd && this._language;
};

SleuthResultsTabManager.prototype.updateView = function()
{
    // array of (DOM ID, disableWhenDebugging?, requireProject?)
    var self = this;
    var buttonIDs = [
        ["projectUnitTestsResults_tabpanel-run-button", 1,
         function() { return self._checkRerunSettings.apply(self); }],
        ["projectUnitTestsResults_tabpanel-stop-button", 0]
    ];
    var busyVal = this.isBusy() ? 1 : 0;
    var testresultsDeck = document.getElementById('test-results-deck');
    var setTabImage;
    if (busyVal) {
        testresultsDeck.selectedIndex = 1;
        //XXX I18N this
        document.getElementById("testresults-running-testplan-name").value =
            "Running " + this._currTestPlanName + " test plan";
        setTabImage = false;
            
    } else if (document.getElementById("testresults-num-passed").value.length > 0) {
        testresultsDeck.selectedIndex = 2;
        setTabImage = true;
    } else {
        testresultsDeck.selectedIndex = 0;
        setTabImage = false;
    }
    if (!setTabImage) {
        document.getElementById('projectUnitTestsResults_tab').setAttribute("results", "none");
    }
    for (var i = 0; i < buttonIDs.length; i++) {
        var btnInfo = buttonIDs[i];
        var button = document.getElementById(btnInfo[0]);
        var enable;
        if (btnInfo[1] == busyVal) {
            enable = false;
        } else if (btnInfo[2]) {
            enable = btnInfo[2]();
        } else {
            enable = true;
        }
        if (enable) {
            if (button.hasAttribute("disabled")) {
                button.removeAttribute("disabled");
            }
        } else {
            button.setAttribute("disabled", "true");
        }        
    }
    var editButton = document.getElementById("projectUnitTestsResults_tabpanel-editTestPlan-button");
    if (this._currTestPlanName) {
        if (editButton.hasAttribute("disabled")) {
            editButton.removeAttribute("disabled");
        }
    } else {
        editButton.setAttribute("disabled", "true");
    }
}

SleuthResultsTabManager.prototype.isReady = function() {
    if (!this._checkRerunSettings()) return false;
    return !this.isBusy();
};

SleuthResultsTabManager.prototype.isBusy = function()
{
    return this._runInProgress;
}

SleuthResultsTabManager.prototype.runStarted = function() {
    this._runInProgress = true;
    this.updateView();
}

SleuthResultsTabManager.prototype.runFinished = function() {
    this._runInProgress = false;
    this.updateView();
};

SleuthResultsTabManager.prototype._pluralize = function(n, wordRoot, plural_suffix, singular_suffix) {
    var s = n + " " + wordRoot;
    if (n == 1) {
        if (typeof(singular_suffix) != "undefined") {
            s += singular_suffix;
        }
    } else if (typeof(plural_suffix) != "undefined") {
        s += plural_suffix;
    }
    return s;   
};

SleuthResultsTabManager.prototype._finishHighlightingLine = function _finishHighlightingLine(lineNumber) {
    this.view.selection.select(lineNumber);
    var treeBoxObject = document.getElementById(tree_id).treeBoxObject;
    treeBoxObject.ensureRowIsVisible(lineNumber);
    this._doSelectTreeLine(lineNumber);
};

SleuthResultsTabManager.prototype.highlightLine = function highlightLine(lineNumber) {
     try {
         if (0 <= lineNumber && lineNumber < this.view.rowCount) {
             this._finishHighlightingLine(lineNumber);
         }
     } catch(ex) {
         log.exception(ex);
     }
};

SleuthResultsTabManager.prototype.moveToNextSimilarResult = function(button_type) {
    try {
        var status =
            document.getElementById("testresults-num-" + button_type).value;
        var num_end = status.indexOf(" ");
        var num = parseInt(status.substr(0, num_end));
        if (num == 0) {
            // This shouldn't happen - the button should be disabled.
            return;
        }
        const expected_url_base = {
            "accept": "passed",
            "exclamation": "failure",
            "error": "error",
            __END__: null
        };
        const treeColumnSelector = {id:"projectUnitTestsResults-filename"};
    
        const numRows = this.view.rowCount;
        var currentIndex = this.view.selection.currentIndex;
        var nextIndex = currentIndex + 1;
        var image_url;
        var basename_re = /([a-z]+)\.png$/;
        while (nextIndex != currentIndex) {
            if (nextIndex >= numRows) {
                nextIndex = 0;
                if (currentIndex == -1) {
                    // There wasn't a selected line at the start, so bail out,
                    // and select the first line.  Normally this is unlikely
                    // to happen.
                    break;
                }
            }
            image_url = this.view.getImageSrc(nextIndex, treeColumnSelector);
            if (basename_re.test(image_url)) {
                var base = RegExp.$1;
                if (base in expected_url_base
                    && expected_url_base[base] == button_type) {
                    break;
                }
            }
            nextIndex += 1;
        }
        // Even if we didn't move anywhere, this ensures that the
        // current line is visible.
        this._finishHighlightingLine(nextIndex);
    }catch(ex) {
        log.exception(ex);
    }
};

SleuthResultsTabManager.prototype._updateButton = function(num, base, suffix)
{
    var label_id = "testresults-num-" + base;
    var button_id = "testresults-button-counter-" + base;
    var labelObj = document.getElementById(label_id);
    labelObj.value = this._pluralize(num, base, suffix);
    
    var button = document.getElementById(button_id);
    button.disabled = (num == 0);
};

SleuthResultsTabManager.prototype.updateResults =
    function(num_passes, num_errors, num_failures, elapsed_time) {
    if (typeof(elapsed_time) == "undefined") {
        elapsed_time = null;
    }
    this._updateButton(num_passes, "passed");
    this._updateButton(num_failures, "failure", "s");
    this._updateButton(num_errors, "error", "s");
    var elapsedTimeBox = document.getElementById("testresults-elapsed-time-container");
    if (elapsed_time != null) {
        document.getElementById("testresults-elapsed-time").value = elapsed_time.toString();
        elapsedTimeBox.hidden = false;
    } else {
        elapsedTimeBox.hidden = true;
    }
    var tab = document.getElementById('projectUnitTestsResults_tab');
    if (num_errors > 0) {
        tab.setAttribute("results", "errors");
    } else if (num_failures > 0) {
        tab.setAttribute("results", "failed");
    } else if (num_passes > 0) {
        tab.setAttribute("results", "passed");
    } else {
        // Map 0/0/0 to an error situation
        tab.setAttribute("results", "errors");
    }
}

SleuthResultsTabManager.prototype.setProject = function(project) {
    if (project == null) {
        this._finishSettingUpGlobalTestPlan();
    } else {
        this._finishSettingUpProject(project);
    }
}

SleuthResultsTabManager.prototype.setDocument = function(document) {
    this._finishSettingUpDocument(document);
}

SleuthResultsTabManager.prototype._switchToProject = function(project) {
    this._project = project;
    this._currTestPlanContainer = project;
    this._currTestPlanContainerType = 'project';
    // Find the test plan associated with this project
    this._switchToContainer();
};

SleuthResultsTabManager.prototype._switchToDocument = function(document) {
    this._document = document;
    if (!document || ko.sleuth.manager.getNumProjectTestPlans(document, 'prefs') == 0) {
        return;
    }
    // Make sure the document has a test plan to switch to
    // Otherwise just update the popup menu
    this._currTestPlanContainer = document;
    this._currTestPlanContainerType = 'document';
    // Find the test plan associated with this document
    this._switchToContainer();
};

SleuthResultsTabManager.prototype._switchToContainer = function() {
    try {
        var containerPrefs = gPrefs.getPref(_gSleuthSvc.LAST_TESTPLAN_PREFNAME);
        var containerID = this._attributeFromContainer('uri');
        if (!containerPrefs.hasStringPref(containerID)) {
            return;
        }
        var entry = containerPrefs.getStringPref(containerID);
        var idx = entry.indexOf(":");
        var lastTestPlanName = entry.substring(idx + 1);
        if (lastTestPlanName) {
            var sleuthPrefs = this._getSleuthPrefs();
            var testPlanNames = ko.sleuth.manager.getTestPlanNamesFromPrefs(sleuthPrefs, true);
            var j = testPlanNames.indexOf(lastTestPlanName);
            if (j > -1) {
                this._currTestPlanName = lastTestPlanName;
            }
        }
    } catch(ex) {
        dump(ex + "\n");
    }
}


SleuthResultsTabManager.prototype._getSleuthPrefs = function(prefContainer, prefContainerType) {
    var prefset = this._attributeFromContainer("prefset", prefContainer, prefContainerType);
    if (prefset.hasPrefHere(_gSleuthSvc.TESTPLAN_PREFNAME)) {
        return prefset.getPref(_gSleuthSvc.TESTPLAN_PREFNAME);
    }
    var sleuthPrefs = Components.classes["@activestate.com/koPreferenceSet;1"].createInstance();
    prefset.setPref(_gSleuthSvc.TESTPLAN_PREFNAME, sleuthPrefs);
    return sleuthPrefs;
}

SleuthResultsTabManager.prototype._finishSettingUpProject = function(project, globalPrefs) {
    this._project = this._currTestPlanContainer = project;
    this._currTestPlanContainerType = 'project';
    return this._finishSettingUpContainer(globalPrefs);
};

SleuthResultsTabManager.prototype._finishSettingUpGlobalTestPlan = function() {
    this._currTestPlanContainer = null;
    this._currTestPlanContainerType = 'project';
    return this._finishSettingUpContainer();
};

SleuthResultsTabManager.prototype._finishSettingUpDocument = function(document) {
    this._document = this._currTestPlanContainer = document;
    this._currTestPlanContainerType = 'document';
    return this._finishSettingUpContainer();
};

SleuthResultsTabManager.prototype._finishSettingUpContainer = function(globalPrefs) {
    if (typeof(globalPrefs) == "undefined") {
        globalPrefs = (Components.classes["@activestate.com/koPrefService;1"].
                       getService(Components.interfaces.koIPrefService).prefs);
    }
    // And figure out if there's a testplan on the project
    try {
        var containerPrefs = globalPrefs.getPref(_gSleuthSvc.LAST_TESTPLAN_PREFNAME);
        var containerID = this._attributeFromContainer('uri');
        // Mark this as our current project
        globalPrefs.setStringPref(_gSleuthSvc.LAST_TESTPLAN_PROJECT_URL, containerID);
        if (!containerPrefs.hasStringPref(containerID)) {
            this._currTestPlanName = null;
            return;
        }
        var entry = containerPrefs.getStringPref(containerID);
        var idx = entry.indexOf(":");
        var testPlanName = entry.substring(idx + 1);
        if (testPlanName && testPlanName != "null") {
            this._currTestPlanName = testPlanName;
        }
        else {
            this._currTestPlanName = null;
        }
    } catch(ex) {
        dump("can't find LAST_TESTPLAN_PREFNAME("
             + this._attributeFromContainer('uri')
             + ") : "
             + ex
             + "\n");
        this._currTestPlanName = null;
    }
    this.updateTestPlanMenu();
};

// Display the current project's test plans, followed by the 
// document's plans, and finally the global plans (to be phased out).

SleuthResultsTabManager.prototype.updateTestPlanMenu = function() {
    var currentProject = this._project = ko.projects.manager.getCurrentProject();
    var currentDocument = this._document = this._getCurrentDocument();
    var sleuthPrefs_Project;
    var sleuthPrefs_Document;
    var sleuthPrefs_Global;
    var sleuthPrefs; // pointer to project or global
    var testPlanNames_Project;
    var testPlanNames_Document;
    var testPlanNames_Global;
    var testPlanNames_all = [];
    var popupmenu = document.getElementById('sleuthResults-menulist-testPlan-popup');
    var menuitem;
    var haveProjectTests = false;
    var haveDocumentTests = false;
    var haveGlobalTests = false;
    
    while (popupmenu.hasChildNodes()) {
        popupmenu.removeChild(popupmenu.lastChild);
    }

    var check_for_match = (this._currTestPlanName != null
                           && (this._currTestPlanContainer == currentProject
                               || this._currTestPlanContainer == currentDocument));
    
    if (currentDocument
        && !currentDocument.prefs.hasPrefHere(_gSleuthSvc.TESTPLAN_PREFNAME)) {
        currentDocument = false;
    }
    var test_base_index = 0;
    this._currTestDocumentBase = 0;
    if (currentDocument) {
        sleuthPrefs_Document = this._getSleuthPrefs(currentDocument, "document");
        testPlanNames_Document = ko.sleuth.manager.getTestPlanNamesFromPrefs(sleuthPrefs_Document,
                                                                            true);
        haveDocumentTests = testPlanNames_Document.length > 0;
        if (haveDocumentTests) {
            this._currTestDocumentBase = test_base_index;
            for (var i = 0; i < testPlanNames_Document.length; i++) {
                testPlanNames_all.push(testPlanNames_Document[i]);
                menuitem = document.createElement("menuitem");
                menuitem.setAttribute("label", testPlanNames_Document[i]);
                var handler = ("ko.sleuth.tabManager._changeTestPlanByIndex" + "("
                               + (test_base_index + i)
                               + ");");
                menuitem.setAttribute("oncommand", handler);
                popupmenu.appendChild(menuitem);
                if (check_for_match
                    && this._currTestPlanName == testPlanNames_Document[i]) {
                    check_for_match = false;
                    currentIndex = i;
                    this._currTestPlanContainer = currentDocument;
                    this._currTestPlanContainerType = 'document';
                    sleuthPrefs = sleuthPrefs_Document;
                }
            }
        }
    }
    
    test_base_index = testPlanNames_all.length;
    this._currTestProjectBase = test_base_index;
    // Don't try to get prefs unless we have them
    if (currentProject
        && !currentProject.prefset.hasPrefHere(_gSleuthSvc.TESTPLAN_PREFNAME)) {
        currentProject = false;
    }
    var currentIndex = -1;
    if (currentProject) {
        sleuthPrefs_Project = this._getSleuthPrefs(currentProject, "project");
        testPlanNames_Project = ko.sleuth.manager.getTestPlanNamesFromPrefs(sleuthPrefs_Project,
                                                                            true);
        haveProjectTests = testPlanNames_Project.length > 0;
        if (haveProjectTests) {
            if (test_base_index > 0) {
                popupmenu.appendChild(document.createElement("menuseparator"));
            }
            for (var i = 0; i < testPlanNames_Project.length; i++) {
                testPlanNames_all.push(testPlanNames_Project[i]);
                menuitem = document.createElement("menuitem");
                menuitem.setAttribute("label", testPlanNames_Project[i]);
                var handler = ("ko.sleuth.tabManager._changeTestPlanByIndex" + "("
                               + (test_base_index + i)
                               + ");");
                menuitem.setAttribute("oncommand", handler);
                popupmenu.appendChild(menuitem);
                if (check_for_match
                    && this._currTestPlanName == testPlanNames_Project[i]) {
                    check_for_match = false;
                    currentIndex = i;
                    this._currTestPlanContainer = currentProject;
                    this._currTestPlanContainerType = 'project';
                    sleuthPrefs = sleuthPrefs_Project;
                }
            }
            test_base_index += testPlanNames_Project.length;
        }
    }

    sleuthPrefs_Global = this._getSleuthPrefs(null, 'global');
    testPlanNames_Global = ko.sleuth.manager.getTestPlanNamesFromPrefs(sleuthPrefs_Global,
                                                                true);
    this._currTestGlobalBase = test_base_index;
    if (testPlanNames_Global.length > 0) {
        haveGlobalTests = true;
        if (test_base_index > 0) {
            popupmenu.appendChild(document.createElement("menuseparator"));
        }

        check_for_match = !!(currentIndex == -1
                             && this._currTestPlanName
                             && !this._currTestPlanContainer);
        for (var i = 0; i < testPlanNames_Global.length; i++) {
            testPlanNames_all.push(testPlanNames_Global[i]);
            menuitem = document.createElement("menuitem");
            menuitem.setAttribute("label", testPlanNames_Global[i]);
            var handler = ("ko.sleuth.tabManager._changeTestPlanByIndex" + "("
                           + (test_base_index + i)
                           + ");");
            menuitem.setAttribute("oncommand", handler);
            popupmenu.appendChild(menuitem);
            if (check_for_match
                && this._currTestPlanName == testPlanNames_Global[i]) {
                currentIndex = i;
                this._currTestPlanContainer = null;
                this._currTestPlanContainerType = 'global';
                sleuthPrefs = sleuthPrefs_Global;
                check_for_match = false;
            }
        }
    }

    this._popupMenu_testPlanNames = testPlanNames_all;

    // Now update the settings
    var menulist_top = document.getElementById("sleuthResults-menulist-testPlan");
    if (testPlanNames_all.length == 0) {
        menuitem = document.createElement("menuitem");
        var label = ("<No test plans defined yet>");
        menuitem.setAttribute("label", label);
        menuitem.setAttribute("disabled", "true");
        popupmenu.appendChild(menuitem);
        this._currTestPlanName = null;
        menulist_top.setAttribute('label', "<None>");
        this._cwd = this._language = this._command_line = null;
    } else {
        if (currentIndex == -1) {
            // Was the last thing global, or a project?
            // Favor document tests first
            if (haveDocumentTests) {
                this._currTestPlanContainer = currentDocument;
                this._currTestPlanContainerType = 'document';
                this._currTestPlanName = testPlanNames_Document[0];
                sleuthPrefs = sleuthPrefs_Document;
            } else if (haveProjectTests) {
                this._currTestPlanContainer = currentProject;
                this._currTestPlanContainerType = 'project';
                this._currTestPlanName = testPlanNames_Project[0];
                sleuthPrefs = sleuthPrefs_Project;
            } else {
                // Fall back to global test plans
                if (!haveGlobalTests) {
                    sleuthResultsLog.error("updateTestPlanMenu: Expected to find a global test, but there is none\n");
                }
                this._currTestPlanContainer = null;
                this._currTestPlanContainerType = 'global';
                this._currTestPlanName = testPlanNames_Global[0];
                sleuthPrefs = sleuthPrefs_Global;
            }
        }
    
        menulist_top.setAttribute('label', this._currTestPlanName);
        // And update the current settings
        try {
            var testPlanPref = sleuthPrefs.getPref(this._currTestPlanName);
            this._setupTestPlanArgumentsFromPrefs(testPlanPref);
        } catch(ex) {
            //dump("Problem getting pref info on test plan ["
            //     + this._currTestPlanName
            //     + "]:\n");
            log.exception(ex);
        }
    }
    this.updateView();
};

SleuthResultsTabManager.prototype._setupTestPlanArgumentsFromPrefs = function(testPlanPref) {
    this._cwd = ko.sleuth.manager.dirFromPrefs(testPlanPref);
    this._language = testPlanPref.getStringPref('language');
    // This check does two things:
    // 1: handle the 4.3 => 4.4 transition, where test plan harnesses named
    // "Perl", "Python", "Ruby", etc. have taken more descriptive names.
    // 2: test plans can be added as extensions.  If a project references an
    // extension on machine A, but the project is moved to machine B where
    // that extension hasn't been installed, we don't try to run it.
    if (!this._harness_indices) {
        this._harness_indices = {};
        var o = {};
        _gSleuthSvc.get_harness_names(o, {});
        for (var name, i = 0; name = o.value[i]; i++) {
            this._harness_indices[name] = i;
        }
    }
    var harness_ok = false;
    if (this._language in this._harness_indices) {
        harness_ok = true;
    } else if (["Perl", "PHP", "Python", "Ruby"].indexOf(this._language) >= 0) {
        // We've got a version 4.3 test harness that will need updating
        if (!this._harness_info_from_language) {
            this._harness_info_from_language = {}
            ko.sleuth.manager.getLangInfoFromHarnessIndices(this._harness_indices,
                                                            this._harness_info_from_language);
        
        }
        if (this._language in this._harness_info_from_language) {
            harness_ok = true;
            // Update this test, dirty the prefset
            var entry = this._harness_info_from_language[this._language];
            testPlanPref.setStringPref("language",
                                       this._language = entry.harnessName);
        }
    }
    if (!harness_ok) {
        sleuthResultsLog.error("Komodo can't find a test harness called "
                               + this._language
                               + "\n");
        // Tests will probably fail now.
    }
    this._command_line = testPlanPref.getStringPref('command_line');
};

SleuthResultsTabManager.prototype._changeTestPlanByIndex = function(i) {
    this._currTestPlanName = this._popupMenu_testPlanNames[i];
    if (i >= this._currTestGlobalBase) {
        this._currTestPlanContainer = null;
        this._currTestPlanContainerType = 'global';
    } else if (i >= this._currTestProjectBase) {
        this._currTestPlanContainer = this._project;
        this._currTestPlanContainerType = 'project';
    } else {
        this._currTestPlanContainer = this._document;
        this._currTestPlanContainerType = 'document';
    }
    try {
        var testPlanPref = this._getSleuthPrefs().getPref(this._currTestPlanName)
        this._setupTestPlanArgumentsFromPrefs(testPlanPref);
    } catch(ex) {
        log.exception(ex);
        // dump("Can't find testplan " + this._currTestPlanName + " in current prefs\n");
    }
};

SleuthResultsTabManager.prototype.clearProject = function() {
    this._currTestPlanContainer = null;
    this._currTestPlanContainerType = 'global';
    this.updateTestPlanMenu();
}

SleuthResultsTabManager.prototype.onClick = function(event)
{
    // sleuthResultsLog.info("SleuthResultsTabManager.onClick(event)");

    // c.f. mozilla/mailnews/base/resources/content/threadPane.js
    var t = event.originalTarget;

    if (t.localName == "treechildren") {
        this._doubleClick();
    }
}

SleuthResultsTabManager.prototype._onKeyPress = function(event)
{
    if (event.charCode) {
        return;
    }
    switch (event.keyCode) {
    case event.DOM_VK_UP:
    case event.DOM_VK_DOWN:
    case event.DOM_VK_PAGE_UP:
    case event.DOM_VK_PAGE_DOWN:
    case event.DOM_VK_HOME:
    case event.DOM_VK_END:
        this._doubleClick();
        break;
    default:
        return;
    }
};


SleuthResultsTabManager.prototype._doubleClick = function()
{
    this._doSelectTreeLine(document.getElementById(tree_id).currentIndex);
};

SleuthResultsTabManager.prototype._doSelectTreeLine = function(rowIndex) {
    var displayPath = this.view.getCellText(rowIndex, {id:"projectUnitTestsResults-context"});
    this._setDetailsText(displayPath);
};

SleuthResultsTabManager.prototype._finishProcessLine = function(line)
{
    var resultsObj = [];
    // Taken from views-buffer.p.xml, onHotSpotDoubleClick
    if (!this.docSvc) {
        this.docSvc = Components.classes['@activestate.com/koDocumentService;1']
                    .getService(Components.interfaces.koIDocumentService);
    }
    this.docSvc.parseHotspotLine(line, this._cwd, {}, resultsObj);
    var results = resultsObj.value;
    return (results.length == 2) ? results : null;
};

SleuthResultsTabManager.prototype._TreeDetail_processLine = function(line)
{
    // Ruby error message hack
    if (this.starts_with_bracket_re.test(line)) {
        line = RegExp.$1;
    }
    var results = this._finishProcessLine(line);
    if (results) {
        var fname = results[0];
        var lineNo = results[1];
        lineNo = Number(lineNo);
        var os = Components.classes["@activestate.com/koOs;1"].createInstance();
        if (os.path.exists(fname)) {
            ko.history.note_curr_loc();
            ko.views.manager.doFileOpenAtLineAsync(ko.uriparse.pathToURI(fname), lineNo);
        } else {
            ko.statusBar.AddMessage("The file '" + fname + "' could not be found.",
                                    'hotspot', 5000, true);
        }
    } else {
        ko.statusBar.AddMessage("Could not parse hotspot line: " + line,
                                'hotspot', 5000, true);
    }
};

}).apply(ko.sleuth);
